#ifndef GLWIDGET_H
#define GLWIDGET_H

#include <QGLWidget>
#include <QString>
#include <QTimer>

typedef struct {
    double x,y,z;
} Point3D;

typedef struct {
    double            time_secs;
    QVector<Point3D>  points;
} MassList;

class MyGLWidget : public QGLWidget
{
    Q_OBJECT

public:
    MyGLWidget(QWidget *parent = 0);
    ~MyGLWidget();

    QSize minimumSizeHint() const;
    QSize sizeHint() const;
    void keyPressEvent( QKeyEvent *keyEvent );

public slots:
    void loadSimulationOutput(void);
    void setYRotationSpeed(int speed);
    void setXRotation(int angle);
    void setYRotation(int angle);
    void setZRotation(int angle);
    void playTime(void);

signals:
    void fileNameChanged(QString s);
    void metaInfoChanged(QString s);
    void versionChanged(QString s);
    void xRotationChanged(int angle);
    void yRotationChanged(int angle);
    void zRotationChanged(int angle);

protected:
    void initializeGL();
    void paintGL();
    void resizeGL(int width, int height);
    void mousePressEvent(QMouseEvent *event);
    void mouseMoveEvent(QMouseEvent *event);
    bool loadSimulationOutputFromFile(const QString &name);
    void computeBounds();

private:
    // Angle of rotation
    int d_xRot;
    int d_yRot;
    int d_zRot;

    // Speed of rotations
    int d_yRotSpeed;

    QPoint d_lastPos;

    // Color palette to use for rendering.
    QColor          d_background;   // Background color
    QVector<QColor> d_colors;   // Colors to use for different model parts
    QVector<bool>   d_visible;  // Is this material visible?

    // Metadata information from the simulation file
    QString    d_metaInfo;

    // Color tags associated with each mass.
    QVector<unsigned>   d_massColors;

    // List of times and set of mass locations.
    QVector<MassList>   d_massLocations;
    int                 d_currentIndex;
    double  d_currentTau;   // Current time step in a double
    double  d_deltaTau;     // How many time steps to advance per render frame

    // Boundary of where any particle has ever gone.
    double  d_minX, d_maxX;
    double  d_minY, d_maxY;
    double  d_minZ, d_maxZ;

    //------------------------------------------------------------
    // Timers needed to take actions
    QTimer  *d_play_timer;
};

#endif
