// forceCollisionsUniformSphere.h
// By Joseph K Aicher, Summer 2014 with Professor Russell M Taylor II

#include "genericForce.h"
#include "chromoTypes.h"

#ifndef FORCECOLLISIONSUNIFORMSPHERE_H
#define FORCECOLLISIONSUNIFORMSPHERE_H



class forceCollisionsUniformSphere : public genericForce
{
protected :
    // structs
    typedef struct POSITION_HASH
    {
        cl_ulong hash;
        cl_ulong index;
        cl_char homeCellType;
        cl_char thisCellType;
        cl_char inCellType; // really 8 bits that will be used like a 8 bool array
    } POSITION_HASH;

    typedef struct COLLISION_CELL
    {
        cl_ulong homeCellStart; // start index of collision cells, starts with home cells
        cl_ulong phantomCellStart; // end index of home cells, start index of phantom cells
        cl_ulong phantomCellEnd; // end index of phantom cells (and thus the group with the same hash)
    } COLLISION_CELL;

    // details about the simulation
    CS_REAL collisionConstant;

    // values of the hashed positions for collision detection
    POSITION_HASH * positionHash;
    // values of the collision cells
    COLLISION_CELL * collisionCell;
    // zero vector
    CS_REAL3 * fZero;

    // openCL details
    cl_kernel clKernelGetHash;
    size_t clKernelGetHashGlobalSize;
    cl_kernel clKernelCollision;
    size_t clKernelCollisionGlobalSize;
    cl_mem clMassRadius;
    cl_mem clBoundingRadius;
    cl_mem clPositionHash;
    cl_mem clCollisionCell;

    cl_device_id deviceId;

    static bool comparisonPOSITION_HASH( const POSITION_HASH & left, const POSITION_HASH & right );


public :
    // constructor
    forceCollisionsUniformSphere (
        std::string sim_CLdir, // the path of directory with openCL source
        cl_context sim_context, // openCL context to use
        cl_command_queue sim_queue, // openCL queue to use
        cl_ulong sim_numParticles, // number of particles in the simulation
        cl_ulong sim_totalMaxInteractions, // total maximum number of interactions (per particle) in Jacobian
        cl_device_id sim_deviceId, // the deviceId we are running on
        cl_mem sim_clPosition, // the positions
        CS_REAL * sim_MassRadius, // the mass radius information
        CS_REAL collisionSpringConstant,
        CS_REAL boundingRadiusScale, // the proportion of bounding radius to mass radius (~ 1.5)
        CS_REAL cellDimensionScale, // the proportion of cell length size to the largest mass radius (~ 5)
        cl_uint gridScale // parameter for hash function: gx + gridScale*( gy + gridScale * gz )
        );

    // destructor
    virtual ~forceCollisionsUniformSphere ( );

    // updates clForce with the collision forces
    virtual void calculateForce ( );

    // gets the name of the force
    virtual const char * getName ( );
};





#endif