// forceCollisionsUniformSphere2.h
// By Joseph K Aicher, Summer 2014 with Professor Russell M Taylor II

#include "genericForce.h"
#include "chromoTypes.h"
// clpp
#include "clpp/clppProgram.h"
#include "clpp/clppContext.h"
#include "clpp/clppSort_RadixSortGPU.h"
#include "clpp/clppSort_RadixSort.h"

#ifndef FORCECOLLISIONSUNIFORMSPHERE2_H
#define FORCECOLLISIONSUNIFORMSPHERE2_H

class forceCollisionsUniformSphere2 : public genericForce
{
protected :

    // details about the simulation
    CS_REAL collisionConstant;

    cl_uint maxHash; // the maximum hash value

    // values of the hashed positions for collision detection
    cl_uint2 * positionHash; // .s[0] is hash, .s[1] is index
    // array of length maxHash with value greater than numParticles
    cl_uint * emptyCollisionCell;

    // openCL details
    // kernel for assigning hashes to each particle
    cl_kernel clKernelGetHash;
    size_t clKernelGetHashGlobalSize;
    // kernel for getting bounds of collision cells in sorted hash array
    cl_kernel clKernelGetBounds;
    size_t clKernelGetBoundsGlobalSize;
    size_t clKernelGetBoundsLocalSize;
    // kernel for using collision cell information to update force information
    cl_kernel clKernelCollision;
    size_t clKernelCollisionGlobalSize;
    cl_kernel clKernelCollisionJacobian;
    size_t clKernelCollisionJacobianGlobalSize;
    // memory objects not inherited from genericForce
    cl_mem clMassRadius;
    cl_mem clPositionHash;
    cl_mem clCollisionCellStart;
    cl_mem clCollisionCellEnd;

    // look up table of good grid scales (hash = (gx + gridScale*(gy + gridScale*gz))%maxHash)
    static const cl_uint lookupGridScale[];
    // function to look up a grid scale that won't be broken by the maxHash, using lookupGridScale
    static cl_int pickGridScale ( cl_uint sim_maxHash );

    // function to perform the first steps of the collision calculations before actually applying the forces
    void helperCalculateForce( );

    static bool comparisonPOSITION_HASH( const cl_uint2 & left, const cl_uint2 & right );

    // sorting using clpp library
    clppContext sortContext;
    clppSort * sortRoutine;


public :
    // constructor
    forceCollisionsUniformSphere2 (
        std::string sim_CLdir, // the path of directory with openCL source
        cl_context sim_context, // openCL context to use
        cl_command_queue sim_queue, // openCL queue to use
        cl_ulong sim_numParticles, // number of particles in the simulation
        cl_ulong sim_totalMaxInteractions, // total maximum number of interactions (per particle) in Jacobian
        cl_platform_id sim_platformId, // the platformId we are running on
        cl_device_id sim_deviceId, // the deviceId we are running on
        cl_mem sim_clPosition, // the positions
        CS_REAL * sim_MassRadius, // the mass radius information
        CS_REAL collisionSpringConstant,
        CS_REAL cellDimensionScale, // the proportion of cell length size to the largest mass radius (~ 2.5)
        cl_uint sim_maxHash // the maximum value of the hash function
        );

    // destructor
    virtual ~forceCollisionsUniformSphere2 ( );

    // updates clForce with the collision forces
    virtual void calculateForce ( );

    // updates both clForce and clJacobian
    virtual void calculateForceAndJacobian ( );

    // gets the name of the force
    virtual const char * getName ( );
};





#endif
