// forceHingesCL.h
// By Joseph K Aicher, Summer 2014 with Professor Russell M Taylor II

#include "genericForce.h"
#include "structureLoader.h"

#ifndef FORCEHINGESCL_H
#define FORCEHINGESCL_H

typedef struct simHINGE {
    cl_ulong m1, m2, m3; // indices for particles hinge attached to
    cl_ulong ndx1, ndx2, ndx3; // indices for force vector
    CS_REAL hingeConstant;
} simHINGE;

class forceHingesCL : public genericForce
{
protected :
    // details about the simulation
    cl_ulong numHinges; // the number of hinges we have

    // openCL details
    cl_mem clHinge;
    cl_kernel clKernelHinge; // calculates hinge forces
    size_t clKernelHingeGlobalSize; // the global size to use with KernelHinge

public :
    // constructor
    forceHingesCL (
        std::string sim_CLdir, // the path of directory with our openCL source code
        cl_context sim_context, // OpenCL context passed in from simulation
        cl_command_queue sim_queue, // OpenCL queue passed in from simulation,
        cl_ulong sim_numParticles, // number of particles in the simulation
        cl_ulong sim_totalMaxInteractions, // total maximum number of interactions (per particle) in Jacobian
        cl_device_id sim_deviceId, // the deviceId we are running on
        cl_mem sim_clPosition, // the openCL pointer for the particles' positions
        HINGE_node * hh // the spring nodes from the configuration (MassMesh)
        );

    // destructor
    virtual ~forceHingesCL ( );

    // updates clForce with hinge forces
    virtual void calculateForce ( );
    
    // gets the name of the force
    virtual const char * getName ( );
};


#endif
