// forceMassDamping.h
// By Joseph K Aicher, Summer 2014 with Professor Russell M Taylor II

#include "genericForce.h"

#ifndef FORCEMASSDAMPING_H
#define FORCEMASSDAMPING_H

class forceMassDamping : public genericForce
{
protected :
    // openCL details
    cl_mem clMassDragConstant; // the drag constants (F = -d * v)
    cl_kernel clKernelMassDrag; // kernel for calculating the forces
    size_t clKernelMassDragGlobalSize;
    // we don't need to have a kernel for calculating the Jacobian
    // because it is constant

public :
    // constructor
    forceMassDamping (
        std::string sim_CLdir, // the path of our directory
        cl_context sim_context, // OpenCL context
        cl_command_queue sim_queue, // OpenCL command queue
        cl_ulong sim_numParticles, // number of particles in simulation
        cl_ulong sim_totalMaxInteractions, // total maximum number of interactions (per particle) in Jacobian
        cl_device_id sim_deviceId, // the deviceId we are running on
        cl_mem sim_clVelocity, // the OpenCL pointer for particles' velocities
        CS_REAL * mass, // the mass of the particles
        CS_REAL * massDamping // the mass damping constants (drag = mass * massDamping)
        );

    // destructor
    virtual ~forceMassDamping ( );

    // updates clForce with drag forces
    virtual void calculateForce ( );

    // updates clForce and clJacobian
    virtual void calculateForceAndJacobian ( );

    // gets the name of the force
    virtual const char * getName ( );
};


#endif