// forceRandomGPU.h
// By Joseph K Aicher, Summer 2014 with Professor Russell M Taylor II

#include "genericForce.h"
#include <random>

#ifndef FORCERANDOMGPU_H
#define FORCERANDOMGPU_H

class forceRandomGPU : public genericForce
{
protected :
    // what is our random force parameter going to be?
    CS_REAL theRandomForce; // multiply our standard normal random variables with this

    // openCL details
    // for random number generation in the kernel
    cl_mem clRandomKey; // the key for our random number generators
    cl_mem clRandomCounter; // the counters for our random number generators
    // the kernel 
    cl_kernel clKernelBrownian; // the kernel that calculates the brownian forces
    size_t clKernelBrownianGlobalSize; // global size for the kernel

    // debugging information about random _integers_ being created
    bool profileIntegers; // have we asked to profile integers
    cl_mem clRandomIntegers; // memory buffer for the int4 values
    cl_kernel clKernelIntegers; // kernel that asks for integers (uses same global size as clKernelBrownian)

public :
    // constructor
    forceRandomGPU (
        std::string sim_CLdir, // the path of directory with our openCL source code
        cl_context sim_context, // OpenCL context passed in from simulation
        cl_command_queue sim_queue, // OpenCL queue passed in from simulation,
        cl_ulong sim_numParticles, // number of particles in the simulation
        cl_ulong sim_totalMaxInteractions, // total maximum number of interactions (per particle) in Jacobian
        CS_REAL sim_timeStep, // the time step in the simulation
        cl_device_id sim_deviceId, // the deviceId we are running on
        CS_REAL randomForce, // the random force parameter from the configuration file
        int randomSeed // seed being used by the random number generator
        );

    // destructor
    ~forceRandomGPU ( );

    // updates clForce with random forces
    virtual void calculateForce ( );
    
    // gets the name of the force
    virtual const char * getName ( );

    // initialize details for debugging integer information
    void initializeProfileIntegers ( );

    // calculate random integers instead of the forces
    void generateIntegers ( );

    // save generated integers to chosen file
    bool saveIntegers ( std::string fileName, bool append );
};

#endif
