// solveIMEXEuler.cl
// By Joseph K Aicher, Summer 2014 with Professor Russell M Taylor II


#include <chromoTypes.cl>


// use IMEX Euler (implicit damping)
__kernel void integrateMotionIMEX(
    const CS_REAL timeStep, // the time step
    __global CS_REAL * inverseMass, // inverse mass values
    __global CS_REAL * dampingScale, // damping scale of all the masses
    __global CS_REAL3 * position, // all the positions we'll update
    __global CS_REAL3 * velocity, // all the velocities we'll update
    __global CS_REAL3 * force, // the force on the particles at this time
    const ulong numParticles // the number of particles
    )
{
    ulong ndx = get_global_id(0); // we can worry about groups later

    if (ndx >= numParticles)
    {
        return;
    }

    // update velocity and position
    CS_REAL3 newv = ( velocity[ndx] + force[ndx] * inverseMass[ndx]
        ) * dampingScale[ndx]; 
    
    position[ndx] = position[ndx] + newv * timeStep;
    velocity[ndx] = newv;
}