// sumForces.h
// By Joseph K Aicher, Summer 2014 with Professor Russell M Taylor II

#include "genericForce.h"
#include <vector>

#ifndef SUMFORCES_H
#define SUMFORCES_H

class sumForces : public genericForce
{
protected :
    // details about the simulation
    std::vector<genericForce*> theForces;

    // OpenCL details
    // keep copies of a zero force and a zero Jacobian on hand to copy to our
    // force and Jacobian
    cl_mem zeroForce;
    cl_mem zeroJacobianRow; // equivalent size to jPosX, jVelY, etc.
    cl_mem zeroJacobianInteractions;

    // kernels for adding the forces and the Jacobians
    std::vector<cl_kernel> addForce;
    std::vector<size_t> addForceGlobalSize;
    std::vector<cl_kernel> addJacobian;
    std::vector<size_t> addJacobianGlobalSize;

    // kernel for finishing the indices of the Jacobian
    cl_kernel kernelFixIndexJacobian; // fixes indices in Jacobians so we don't have two entries for same particle
    size_t kernelFixIndexJacobianGlobalSize;

    void helperAddForces ( ); // adds the forces after calculating each of the forces


public :
    // constructor
    sumForces ( 
        std::string sim_CLdir, // the path of directory with our openCL source code
        cl_context sim_context, // OpenCL context passed in from simulation
        cl_command_queue sim_queue, // OpenCL queue passed in from simulation
        cl_ulong sim_numParticles, // number of particles in simulation
        cl_ulong sim_totalMaxInteractions, // total maximum number of interactions (per particle) in Jacobian
        cl_device_id sim_deviceId, // the deviceId we are running on
        const std::vector<genericForce*> & sim_forces // vector of the forces to add up
        );

    // destructor
    virtual ~sumForces ( );

    // update clForce with the sum of the forces, sets updated to false
    virtual void calculateForce ( );

    // update clForce and clJacobian, sets updated to false
    virtual void calculateForceAndJacobian ( );

    // get the name of this force
    virtual const char * getName ( );
};


#endif